package org.greenstone.gatherer.collection;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.greenstone.gatherer.Gatherer;
/** Provides a prompt allowing the user some choice in whether a collection saves. */
public class SaveCollectionBox
    extends JDialog
    implements ActionListener{
    /** What option the user has choosen. */
    private int result = 0;
    /** Button to cancel prompt, no save. */
    private JButton cancel = null;
    /** Button for no save. */
    private JButton no = null;
    /** Button to save. */
    private JButton yes = null;
    /** A reference to ourselves so our inner classes can dispose of us. */
    private SaveCollectionBox myself = null;
    /** Result value if the user has choosen cancel. */
    static final public int SAVE_CANCEL  = 0;
    /** Result value if the user has choosen no. */
    static final public int SAVE_NO      = 1;
    /** Result value if the user has choosen yes. */
    static final public int SAVE_YES     = 2;
    /** Construtor. */
    public SaveCollectionBox() {
	super(Gatherer.g_man);
	this.myself = this;
	result = SAVE_CANCEL;
	// Dialog setup
	this.setModal(true);
	this.setTitle(get("Title"));
	this.setSize(360,100);
    }
    /** Any implementation of <i>ActionListener</i> must include this method so that we can be informed when an action has occured. In this case we see what button the users clicked, set the appropriate result then dispose of the dialog. */
    public void actionPerformed(ActionEvent event) {
	if(event.getSource() == yes) {
	    result = SAVE_YES;
	    this.dispose();
	} else if(event.getSource() == no) {
	    result = SAVE_NO;
	    this.dispose();
	} else if(event.getSource() == cancel) {
	    result = SAVE_CANCEL;
	    this.dispose();
	}
    }
    /** Destructor. */
    public void destroy() {
	cancel = null;
	no.removeActionListener(this);
	no = null;
	yes.removeActionListener(this);
	yes = null;
	myself = null;
	rootPane = null;
    }
    /** Displays the prompt by first building the controls, then waits for a user selection.
	  * @param name The name of the current collection as a <strong>String</strong>.
	  */
    public int getUserOption(String name) {
	JPanel content_pane = (JPanel) this.getContentPane();
	content_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	content_pane.setLayout(new GridLayout(2,1));

	String args[] = new String[1];
	args[0] = name;
	JLabel save_label = new JLabel(get("Label", args));
	content_pane.add(save_label);
	JPanel button_pane = new JPanel(new GridLayout(1,3));
	content_pane.add(button_pane);
	// We add both mnemonics and key listener so that the 'y' of yes is underlined, but pressing just [Y] (rather than [CTL]-[Y]) performs a systematic click.
	yes = new JButton(get("General.Yes"));
	yes.addActionListener(this);
	KeyListenerImpl key_listener = new KeyListenerImpl(); 
	yes.addKeyListener(key_listener);
	yes.setMnemonic(KeyEvent.VK_Y);
	button_pane.add(yes);
	no = new JButton(get("General.No"));
	no.addActionListener(this);
	no.addKeyListener(key_listener);
	no.setMnemonic(KeyEvent.VK_N);
	button_pane.add(no);
	cancel = new JButton(get("General.Cancel"));
	cancel.addActionListener(this);
	cancel.addKeyListener(key_listener);
	cancel.setMnemonic(KeyEvent.VK_C);
	button_pane.add(cancel);
	// Center on screen.
	Dimension dlgSize = getSize();
	Dimension frmSize = Gatherer.g_man.getSize();
	Point loc = Gatherer.g_man.getLocation();
	setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
	show();
	// Deallocate things we allocated here.
	yes.removeKeyListener(key_listener);
	no.removeKeyListener(key_listener);
	cancel.removeKeyListener(key_listener);
	key_listener = null;
	content_pane = null;
	args = null;
	save_label = null;
	button_pane = null;
	dlgSize = null;
	frmSize = null;
	loc = null;
	return result;
    }
    /** Method to retrieve a phrase from the dictionary based of a key.
	  * @param key A <strong>String</strong> used to find the correct phrase.
	  * @return A <strong>String</strong> containing the correct phrase with the correct formatting.
	  */
    private String get(String key) {
	return get(key, null);
    }
    /** Method to retrieve a phrase from the dictionary based of a key and including arguments.
	  * @param key A <strong>String</strong> used to find the correct phrase.
	  * @param args A <strong>String[]</strong> of arguments used in formatting and filling out the phrase.
	  * @return A <strong>String</strong> containing the correct phrase with the correct formatting.
	  */
    private String get(String key, String args[]) {
	if(key.indexOf('.') == -1) {
	    key = "SaveCollectionBox." + key;
	}
	return Gatherer.dictionary.get(key, args);
    }
    /** Listens for key presses when the focus is on one of the buttons. Note that the key pressed needn't be the one associated with the button in focus. */
    private class KeyListenerImpl
	extends KeyAdapter {
	/** Any extension of KeyAdapter can override this method so that we can be informed whenever a key is released (ie after a keyTyped event). In this case we map the key press to an appropriate click on one of the buttons (not necessarily the one the key release was detected on).
	 * @param event A <strong>KeyEvent</strong> containing details about the key release event.
			*/
	public void keyReleased(KeyEvent event) {
	    if(event.getKeyCode() == KeyEvent.VK_Y) {
		yes.doClick(10);
	    }
	    else if(event.getKeyCode() == KeyEvent.VK_N) {
		no.doClick(10);
	    }
	    else if(event.getKeyCode() == KeyEvent.VK_C) {
		cancel.doClick(10);
	    }
	}
    }
}
